@@ -1,12 +1,17 @@
+import canvasapi
 from canvasapi import Canvas
 from canvasapi.exceptions import Forbidden, InvalidAccessToken, ResourceDoesNotExist
 
 from canvas_course_tools.datatypes import (
+    Assignment,
+    AssignmentGroup,
+    Attachment,
     Course,
+    Group,
+    GroupSet,
     Section,
     Student,
-    GroupSet,
-    Group,
+    Submission,
 )
 
 
@@ -34,15 +39,7 @@ class CanvasTasks:
             A list of Canvas course objects.
         """
         courses = self.canvas.get_courses(include=["term"])
-        return [
-            Course(
-                id=course.id,
-                name=course.name,
-                course_code=course.course_code,
-                term=course.term["name"],
-            )
-            for course in courses
-        ]
+        return [create_course_object(course) for course in courses]
 
     def get_course(self, course_id):
         """Get a Canvas course by id.
@@ -54,12 +51,7 @@ class CanvasTasks:
             A Canvas course object.
         """
         course = self.canvas.get_course(course_id, include=["term"])
-        return Course(
-            id=course.id,
-            name=course.name,
-            course_code=course.course_code,
-            term=course.term["name"],
-        )
+        return create_course_object(course)
 
     def get_students(self, course_id):
         """Get all students in a course.
@@ -72,14 +64,7 @@ class CanvasTasks:
         """
         course = self.canvas.get_course(course_id)
         students = course.get_users(enrollment_type=["student"])
-        return [
-            Student(
-                id=student.id,
-                name=student.short_name,
-                sortable_name=student.sortable_name,
-            )
-            for student in students
-        ]
+        return [create_student_object(student) for student in students]
 
     def get_sections(self, course_id):
         """Get a list of sections, including students
@@ -133,6 +118,33 @@ class CanvasTasks:
         groupset = canvas_course.create_group_category(name)
         return GroupSet(id=groupset.id, name=name)
 
+    def list_groupsets(self, course: Course) -> list[GroupSet]:
+        """List groupsets in a course.
+
+        Args:
+            course (Course): the course containing the groupsets.
+
+        Returns:
+            list[GroupSet]: a list of GroupSet objects.
+        """
+        groupsets = course._course.get_group_categories()
+        return [
+            GroupSet(id=groupset.id, name=groupset.name, _group_set=groupset)
+            for groupset in groupsets
+        ]
+
+    def get_groupset(self, group_set_id: int) -> GroupSet:
+        """Get a groupset by id
+
+        Args:
+            group_set_id (int): the groupset ID.
+
+        Returns:
+            GroupSet: the requests groupset object.
+        """
+        groupset = self.canvas.get_group_category(group_set_id)
+        return GroupSet(id=groupset.id, name=groupset.name, _group_set=groupset)
+
     def create_group(self, group_name, group_set):
         """Create a group inside a GroupSet.
 
@@ -148,6 +160,10 @@ class CanvasTasks:
         group = groupset.create_group(name=group_name)
         return Group(id=group.id, name=group_name)
 
+    def list_groups(self, group_set: GroupSet) -> list[Group]:
+        groups = group_set._group_set.get_groups()
+        return [Group(id=group.id, name=group.name, _group=group) for group in groups]
+
     def add_student_to_group(self, student, group):
         """Add student to a group.
 
@@ -157,3 +173,54 @@ class CanvasTasks:
         """
         canvas_group = self.canvas.get_group(group.id)
         canvas_group.create_membership(student.id)
+
+    def get_students_in_group(self, group: Group) -> list[Student]:
+        students = group._group.get_users()
+        return [create_student_object(student) for student in students]
+
+    def get_assignment_groups(self, course: Course) -> list[AssignmentGroup]:
+        groups = course._course.get_assignment_groups()
+        return [
+            AssignmentGroup(id=group.id, name=group.name, course=course)
+            for group in groups
+        ]
+
+    def get_assignments_for_group(self, group: AssignmentGroup) -> list[Assignment]:
+        assignments = group.course._course.get_assignments_for_group(group.id)
+        return [
+            Assignment(
+                id=assignment.id,
+                name=assignment.name,
+                submission_types=assignment.submission_types,
+                _api=assignment,
+            )
+            for assignment in assignments
+        ]
+
+    def get_submission(self, assignment: Assignment, student: Student) -> Submission:
+        submission = assignment._api.get_submission(user=student.id)
+        attachments = [
+            Attachment(
+                name=a.filename, url=a.url, content_type=getattr(a, "content-type")
+            )
+            for a in submission.attachments
+        ]
+        return Submission(student=student, attachments=attachments)
+
+
+def create_course_object(course: canvasapi.course.Course):
+    return Course(
+        id=course.id,
+        name=course.name,
+        course_code=course.course_code,
+        term=course.term["name"],
+        _course=course,
+    )
+
+
+def create_student_object(student: canvasapi.user.User):
+    return Student(
+        id=student.id,
+        name=student.short_name,
+        sortable_name=student.sortable_name,
+    )
